/*  ttk91vmgui
 *  Copyright (C) 1999 Kimmo.palin@helsinki.fi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include<stdio.h>
#include <gtk/gtk.h>
#include<gtk/gtklabel.h>

#include<getopt.h>
#include "cpu_access.h"
#include "compiler.h"
#include "cpu.h"
#include "mem_access.h"
#include "gladesrc.h"
#include "gladesig.h"

  GtkWidget *mainWin;
  GtkWidget *fileLoadDialog;
  GtkWidget *memWin;
  GtkWidget *symWin;
  GtkWidget *kbdDialog;
  GtkWidget *crtDialog;
  GtkWidget *errDialog;

cpudev *mainCPU;
GtkCList *GtkMemList;
GtkCList *GtkSymList;
GtkCList *GtkRegList;
GtkCList *GtkCPURegList;

void add_regs();

extern int yydebug;
extern int yy_flex_debug;

gint run_timeout_tag;
gboolean strait_run=FALSE;


void print_usage(char *pname)
{
  fprintf(stderr,"usage: %s [-h] [-p] [-l] [-m num]\n",pname);
}


int
main (int argc, char *argv[])
{


  int cores=DEF_CORE,coml;

  yy_flex_debug=0;
  yydebug=0;

  gtk_set_locale ();

  gtk_init (&argc, &argv);


  while((coml=getopt(argc,argv,"hm:pl"))!=EOF) {
    switch(coml) {
    case 'm':
      cores=atoi(optarg);
      break;
    case 'p':
      yydebug=1;
      break;
    case 'l':
      yy_flex_debug=1;
      break;
    case 'h':
    default:
      print_usage(argv[0]);
      break;
    }
  }

  
  /*
   * The following code was added by Glade to create one of each component
   * (except popup menus), just so that you see something after building
   * the project. Delete any components that you don't want shown initially.
   */
  fileLoadDialog = create_fileLoadDialog ();

  memWin = create_memWin ();
  symWin = create_symWin ();

  kbdDialog = create_kbdDialog ();
  crtDialog = create_crtDialog ();
  errDialog = create_errDialog ();
  mainWin = create_mainWin ();

  GtkMemList=GTK_CLIST(get_widget(memWin,"memList"));
  GtkSymList=GTK_CLIST(get_widget(symWin,"symList"));
  GtkRegList=GTK_CLIST(get_widget(mainWin,"regList"));
  GtkCPURegList=GTK_CLIST(get_widget(mainWin,"cpuRegList"));

#ifdef _AUTO_SCROLLBARS
  gtk_clist_set_policy(GtkRegList,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  gtk_clist_set_policy(GtkSymList,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  gtk_clist_set_policy(GtkMemList,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
  gtk_clist_set_policy(GtkCPURegList,GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
#endif

  init_core(cores);


  mainCPU=init_cpu();

  add_regs();

  set_message("TTK91VM For Linux");

  gtk_widget_show (symWin);
  gtk_widget_show (memWin);
  gtk_widget_show (mainWin);

  gtk_main ();
  return 0;
}

void pause_run()
{
  if(strait_run)
    gtk_timeout_remove(run_timeout_tag);
}

void continue_run()
{
  if(strait_run)
    run_timeout_tag=gtk_timeout_add(1000,process_timeout,mainCPU);

}

void add_regs()
{
  int i;
  GtkWidget *listw=get_widget(mainWin,"regList");
  gchar *regNames[12][2]={{"R0",0},{"R1",0},{"R2",0},{"R3",0},{"R4",0},{"R5",0},
			  {"SP",0},{"FP",0},
			  {"PC",0},{"TR",0},{"SR",0},{"IR",0}};
  gchar strbuf[15];

  for(i=0;i<8;i++) {
    regNames[i][1]=strbuf;
    sprintf((char*)strbuf,"%d",get_reg(mainCPU,i));
    gtk_clist_append(GTK_CLIST(listw),regNames[i]);
  }
  listw=get_widget(mainWin,"cpuRegList");
  regNames[i][1]=strbuf;
  sprintf((char*)regNames[1],"%d",get_pc(mainCPU));
  gtk_clist_append(GTK_CLIST(listw),regNames[i++]);
  regNames[i][1]=strbuf;
  sprintf((char*)regNames[1],"%d",get_tr(mainCPU));
  gtk_clist_append(GTK_CLIST(listw),regNames[i++]);
  regNames[i][1]=strbuf;
  sprintf((char*)regNames[1],"%d",get_sr(mainCPU));
  gtk_clist_append(GTK_CLIST(listw),regNames[i++]);
  regNames[i][1]=strbuf;
  sprintf((char*)regNames[1],"%d",get_ir(mainCPU));
  gtk_clist_append(GTK_CLIST(listw),regNames[i++]);

}


void set_message(char *msg)
{
  GtkStatusbar *sbar=GTK_STATUSBAR(get_widget(mainWin,"statusbar1"));
  gint bar_ctx=gtk_statusbar_get_context_id(sbar,"TTK91vm");

  gtk_statusbar_pop(sbar,bar_ctx);
  gtk_statusbar_push(sbar,bar_ctx,msg);

  while (gtk_events_pending())
    gtk_main_iteration();

}

void
on_loadFileMenu_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_widget_show(fileLoadDialog);
}


void
on_exitMenu_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


void
on_showMemMenu_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if(GTK_CHECK_MENU_ITEM(menuitem)->active)
    gtk_widget_show(memWin);
  else
    gtk_widget_hide(memWin);

}


void
on_showSymMenu_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  if(GTK_CHECK_MENU_ITEM(menuitem)->active)
    gtk_widget_show(symWin);
  else
    gtk_widget_hide(symWin);

}


void
on_cpuRun_clicked                      (GtkButton       *button,
                                        gpointer         user_data)
{
  static gboolean ajossa=FALSE;

  if(ajossa) {
    pause_run();
    set_message("Ohjelma pysytetty!");
    ajossa=FALSE;
  } else {
    set_message("Ajetaan ohjelmaa!");
    strait_run=TRUE;
    ajossa=TRUE;
    continue_run();
  }
}


void
on_cpuStep_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  memunit tmp;

  set_message("Askella!");
  strait_run=FALSE;
  if(getCoreSize()>get_pc(mainCPU) && 
     (get_tr(mainCPU)!=S_HALT || (get_sr(mainCPU)&STAT_S)==0) && 
     (get_sr(mainCPU)&(STAT_U|STAT_M))==0) {
    tmp=getMem(mainCPU,get_pc(mainCPU));
    set_ir(mainCPU,tmp.flds);
    set_pc(mainCPU,get_pc(mainCPU)+1);

    exec_instr(mainCPU);
  }
}


void
on_crtOK_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(crtDialog);
  continue_run();
}


void
on_fileLoadOk_clicked                  (GtkButton       *button,
                                        gpointer         user_data)
{
  char *apu,buffer[512];

  gtk_widget_hide(fileLoadDialog);
  set_sr(mainCPU,0);
  apu=gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileLoadDialog));
  sprintf(buffer,"tiedosto: %s",apu);
  set_message(buffer);
  

  if(comp_file(mainCPU,apu)==0) {
    gtk_add_symbols();
    set_reg(mainCPU,memlimit-1,SPREG);
    set_reg(mainCPU,memlimit-1,FPREG);
    set_pc(mainCPU,0);
    /*sprintf(buffer,"Knnetty: %s",apu);
    set_message(buffer);*/
  }
}

static gint button_press_callback( GtkWidget      *widget, 
				   GdkEventButton *event,
				   gpointer        data )
{
  gtk_grab_remove(kbdDialog);
  gtk_widget_hide(kbdDialog);

  return TRUE;
}

void kbd_input()
{

  pause_run();
  gtk_widget_show(kbdDialog);
  gtk_grab_add(kbdDialog);
  /*  gtk_signal_connect(GTK_OBJECT(get_widget(kbdDialog,"kbdOk")),
		     "button_press_event",
		     GTK_SIGNAL_FUNC(button_press_callback), 
  	     NULL);
  */

}
  
    
void err_message(char *message) 
{
  GtkLabel *errMesg=GTK_LABEL(get_widget(errDialog,"errValue"));
  gtk_label_set(errMesg,message);
  gtk_widget_show(errDialog);
}

void crt_message(int value) 
{
  char message[128];
  GtkLabel *crtMesg=GTK_LABEL(get_widget(crtDialog,"crtValue"));

  pause_run();

  sprintf(message,"%d",value);
  gtk_label_set(crtMesg,message);
  gtk_widget_show(crtDialog);
}



gboolean
on_mainWin_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  gtk_main_quit();
  return TRUE;
}


void
on_fileLoadCancel_clicked              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(fileLoadDialog);
}


gboolean
on_memWin_delete_event                 (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GtkWidget *wmenu=get_widget(mainWin,"showMemMenu");
  gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(wmenu),FALSE);

  return TRUE;
}


gboolean
on_symWin_delete_event                 (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  GtkWidget *wmenu=get_widget(mainWin,"showSymMenu");
  gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(wmenu),FALSE);

  return TRUE;
}

void
on_kbdOk_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
  char *entrystr;

  gtk_grab_remove(kbdDialog);
  gtk_widget_hide(kbdDialog);
  entrystr=gtk_entry_get_text(GTK_ENTRY(get_widget(kbdDialog,"kbdValue")));
  kbd_input_done(atoi(entrystr));
  continue_run();

}



void
on_errOK_clicked                       (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_hide(errDialog);
}
