
/*  ttk91vmgui
 *  Copyright (C) 1999 kimmo.palin@helsinki.fi
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gladesig.h"
#include "gladesrc.h"

GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("../../pixmaps/", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_mainWin ()
{
  GtkWidget *mainWin;
  GtkWidget *vbox1;
  GtkWidget *menubar1;
  GtkWidget *fileMenu;
  GtkWidget *fileMenu_menu;
  GtkWidget *loadFileMenu;
  GtkWidget *separator1;
  GtkWidget *exitMenu;
  GtkWidget *winMenu;
  GtkWidget *winMenu_menu;
  GtkWidget *showMemMenu;
  GtkWidget *showSymMenu;
  GtkWidget *hbox1;
  GtkWidget *regList;
  GtkWidget *regName;
  GtkWidget *regVal;
  GtkWidget *vbox2;
  GtkWidget *cpuRegList;
  GtkWidget *cpuRegName;
  GtkWidget *cpuRegVal;
  GtkWidget *vbox3;
  GtkWidget *cpuRun;
  GtkWidget *cpuStep;
  GtkWidget *statusbar1;
  GtkAcceleratorTable *accelerator_table;
  GtkTooltips *tooltips;

  tooltips = gtk_tooltips_new ();

  mainWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (mainWin, "mainWin");
  gtk_object_set_data (GTK_OBJECT (mainWin), "mainWin", mainWin);
  gtk_widget_set_usize (mainWin, 460, 400);
  gtk_signal_connect (GTK_OBJECT (mainWin), "delete_event",
                      GTK_SIGNAL_FUNC (on_mainWin_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (mainWin), "TTK91vm for Linux");
  gtk_window_set_policy (GTK_WINDOW (mainWin), TRUE, TRUE, FALSE);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_object_set_data (GTK_OBJECT (mainWin), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (mainWin), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_widget_set_name (menubar1, "menubar1");
  gtk_object_set_data (GTK_OBJECT (mainWin), "menubar1", menubar1);
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, TRUE, 0);

  fileMenu = gtk_menu_item_new_with_label ("Tiedosto");
  gtk_widget_set_name (fileMenu, "fileMenu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "fileMenu", fileMenu);
  gtk_widget_show (fileMenu);
  gtk_container_add (GTK_CONTAINER (menubar1), fileMenu);

  fileMenu_menu = gtk_menu_new ();
  gtk_widget_set_name (fileMenu_menu, "fileMenu_menu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "fileMenu_menu", fileMenu_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (fileMenu), fileMenu_menu);

  loadFileMenu = gtk_menu_item_new_with_label ("Lataa");
  gtk_widget_set_name (loadFileMenu, "loadFileMenu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "loadFileMenu", loadFileMenu);
  gtk_widget_show (loadFileMenu);
  gtk_container_add (GTK_CONTAINER (fileMenu_menu), loadFileMenu);
  gtk_signal_connect (GTK_OBJECT (loadFileMenu), "activate",
                      GTK_SIGNAL_FUNC (on_loadFileMenu_activate),
                      NULL);

  separator1 = gtk_menu_item_new ();
  gtk_widget_set_name (separator1, "separator1");
  gtk_object_set_data (GTK_OBJECT (mainWin), "separator1", separator1);
  gtk_widget_show (separator1);
  gtk_container_add (GTK_CONTAINER (fileMenu_menu), separator1);

  exitMenu = gtk_menu_item_new_with_label ("Poistu");
  gtk_widget_set_name (exitMenu, "exitMenu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "exitMenu", exitMenu);
  gtk_widget_show (exitMenu);
  gtk_container_add (GTK_CONTAINER (fileMenu_menu), exitMenu);
  gtk_signal_connect (GTK_OBJECT (exitMenu), "activate",
                      GTK_SIGNAL_FUNC (on_exitMenu_activate),
                      NULL);

  winMenu = gtk_menu_item_new_with_label ("Ikkunat");
  gtk_widget_set_name (winMenu, "winMenu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "winMenu", winMenu);
  gtk_widget_show (winMenu);
  gtk_container_add (GTK_CONTAINER (menubar1), winMenu);

  winMenu_menu = gtk_menu_new ();
  gtk_widget_set_name (winMenu_menu, "winMenu_menu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "winMenu_menu", winMenu_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (winMenu), winMenu_menu);

  showMemMenu = gtk_check_menu_item_new_with_label ("Nyt muisti");
  gtk_widget_set_name (showMemMenu, "showMemMenu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "showMemMenu", showMemMenu);
  gtk_widget_show (showMemMenu);
  gtk_container_add (GTK_CONTAINER (winMenu_menu), showMemMenu);
  gtk_signal_connect (GTK_OBJECT (showMemMenu), "activate",
                      GTK_SIGNAL_FUNC (on_showMemMenu_activate),
                      NULL);
  gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (showMemMenu), TRUE);

  showSymMenu = gtk_check_menu_item_new_with_label ("Nyt symbolit");
  gtk_widget_set_name (showSymMenu, "showSymMenu");
  gtk_object_set_data (GTK_OBJECT (mainWin), "showSymMenu", showSymMenu);
  gtk_widget_show (showSymMenu);
  gtk_container_add (GTK_CONTAINER (winMenu_menu), showSymMenu);
  gtk_signal_connect (GTK_OBJECT (showSymMenu), "activate",
                      GTK_SIGNAL_FUNC (on_showSymMenu_activate),
                      NULL);
  gtk_check_menu_item_set_state (GTK_CHECK_MENU_ITEM (showSymMenu), TRUE);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox1, "hbox1");
  gtk_object_set_data (GTK_OBJECT (mainWin), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, TRUE, TRUE, 0);

  regList = gtk_clist_new (2);
  gtk_widget_set_name (regList, "regList");
  gtk_object_set_data (GTK_OBJECT (mainWin), "regList", regList);
  gtk_widget_show (regList);
  gtk_box_pack_start (GTK_BOX (hbox1), regList, TRUE, TRUE, 0);
  gtk_clist_set_column_width (GTK_CLIST (regList), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (regList), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (regList));

  regName = gtk_label_new ("Nimi");
  gtk_widget_set_name (regName, "regName");
  gtk_object_set_data (GTK_OBJECT (mainWin), "regName", regName);
  gtk_widget_show (regName);
  gtk_clist_set_column_widget (GTK_CLIST (regList), 0, regName);

  regVal = gtk_label_new ("Arvo");
  gtk_widget_set_name (regVal, "regVal");
  gtk_object_set_data (GTK_OBJECT (mainWin), "regVal", regVal);
  gtk_widget_show (regVal);
  gtk_clist_set_column_widget (GTK_CLIST (regList), 1, regVal);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox2, "vbox2");
  gtk_object_set_data (GTK_OBJECT (mainWin), "vbox2", vbox2);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox2, TRUE, TRUE, 0);

  cpuRegList = gtk_clist_new (2);
  gtk_widget_set_name (cpuRegList, "cpuRegList");
  gtk_object_set_data (GTK_OBJECT (mainWin), "cpuRegList", cpuRegList);
  gtk_widget_show (cpuRegList);
  gtk_box_pack_start (GTK_BOX (vbox2), cpuRegList, TRUE, TRUE, 0);
  gtk_clist_set_column_width (GTK_CLIST (cpuRegList), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (cpuRegList), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (cpuRegList));

  cpuRegName = gtk_label_new ("Nimi");
  gtk_widget_set_name (cpuRegName, "cpuRegName");
  gtk_object_set_data (GTK_OBJECT (mainWin), "cpuRegName", cpuRegName);
  gtk_widget_show (cpuRegName);
  gtk_clist_set_column_widget (GTK_CLIST (cpuRegList), 0, cpuRegName);

  cpuRegVal = gtk_label_new ("Arvo");
  gtk_widget_set_name (cpuRegVal, "cpuRegVal");
  gtk_object_set_data (GTK_OBJECT (mainWin), "cpuRegVal", cpuRegVal);
  gtk_widget_show (cpuRegVal);
  gtk_clist_set_column_widget (GTK_CLIST (cpuRegList), 1, cpuRegVal);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox3, "vbox3");
  gtk_object_set_data (GTK_OBJECT (mainWin), "vbox3", vbox3);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox3, TRUE, TRUE, 0);

  cpuRun = gtk_button_new_with_label ("Aja/Pysyt");
  gtk_widget_set_name (cpuRun, "cpuRun");
  gtk_object_set_data (GTK_OBJECT (mainWin), "cpuRun", cpuRun);
  gtk_widget_show (cpuRun);
  gtk_box_pack_start (GTK_BOX (vbox3), cpuRun, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (cpuRun), 20);
  gtk_tooltips_set_tip (tooltips, cpuRun, "Aja ohjelma alusta loppuun", NULL);
  gtk_signal_connect (GTK_OBJECT (cpuRun), "clicked",
                      GTK_SIGNAL_FUNC (on_cpuRun_clicked),
                      NULL);

  cpuStep = gtk_button_new_with_label ("Askella");
  gtk_widget_set_name (cpuStep, "cpuStep");
  gtk_object_set_data (GTK_OBJECT (mainWin), "cpuStep", cpuStep);
  gtk_widget_show (cpuStep);
  gtk_box_pack_start (GTK_BOX (vbox3), cpuStep, TRUE, TRUE, 0);
  gtk_container_border_width (GTK_CONTAINER (cpuStep), 20);
  GTK_WIDGET_SET_FLAGS (cpuStep, GTK_CAN_DEFAULT);
  gtk_widget_grab_focus (cpuStep);
  gtk_widget_grab_default (cpuStep);
  gtk_tooltips_set_tip (tooltips, cpuStep, "Suorita seuraava ksky", NULL);
  gtk_signal_connect (GTK_OBJECT (cpuStep), "clicked",
                      GTK_SIGNAL_FUNC (on_cpuStep_clicked),
                      NULL);
  accelerator_table = gtk_accelerator_table_new ();
  gtk_window_add_accelerator_table (GTK_WINDOW (mainWin), accelerator_table);
  gtk_widget_install_accelerator (cpuStep, accelerator_table, "clicked",
                                  GDK_space, 0);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_set_name (statusbar1, "statusbar1");
  gtk_object_set_data (GTK_OBJECT (mainWin), "statusbar1", statusbar1);
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, TRUE, 0);

  gtk_object_set_data (GTK_OBJECT (mainWin), "tooltips", tooltips);

  return mainWin;
}

GtkWidget*
create_fileLoadDialog ()
{
  GtkWidget *fileLoadDialog;
  GtkWidget *fileLoadOk;
  GtkWidget *fileLoadCancel;

  fileLoadDialog = gtk_file_selection_new ("Lataa");
  gtk_widget_set_name (fileLoadDialog, "fileLoadDialog");
  gtk_object_set_data (GTK_OBJECT (fileLoadDialog), "fileLoadDialog", fileLoadDialog);
  gtk_container_border_width (GTK_CONTAINER (fileLoadDialog), 10);

  fileLoadOk = GTK_FILE_SELECTION (fileLoadDialog)->ok_button;
  gtk_widget_set_name (fileLoadOk, "fileLoadOk");
  gtk_object_set_data (GTK_OBJECT (fileLoadDialog), "fileLoadOk", fileLoadOk);
  gtk_widget_show (fileLoadOk);
  GTK_WIDGET_SET_FLAGS (fileLoadOk, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (fileLoadOk), "clicked",
                      GTK_SIGNAL_FUNC (on_fileLoadOk_clicked),
                      NULL);

  fileLoadCancel = GTK_FILE_SELECTION (fileLoadDialog)->cancel_button;
  gtk_widget_set_name (fileLoadCancel, "fileLoadCancel");
  gtk_object_set_data (GTK_OBJECT (fileLoadDialog), "fileLoadCancel", fileLoadCancel);
  gtk_widget_show (fileLoadCancel);
  GTK_WIDGET_SET_FLAGS (fileLoadCancel, GTK_CAN_DEFAULT);
  gtk_signal_connect (GTK_OBJECT (fileLoadCancel), "clicked",
                      GTK_SIGNAL_FUNC (on_fileLoadCancel_clicked),
                      NULL);

  return fileLoadDialog;
}

GtkWidget*
create_memWin ()
{
  GtkWidget *memWin;
  GtkWidget *memList;
  GtkWidget *memAddrLbl;
  GtkWidget *memNumVal;
  GtkWidget *memIstrAddr;

  memWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (memWin, "memWin");
  gtk_object_set_data (GTK_OBJECT (memWin), "memWin", memWin);
  gtk_widget_set_usize (memWin, 300, 400);
  gtk_signal_connect (GTK_OBJECT (memWin), "delete_event",
                      GTK_SIGNAL_FUNC (on_memWin_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (memWin), "Muisti");
  gtk_window_set_policy (GTK_WINDOW (memWin), TRUE, TRUE, FALSE);

  memList = gtk_clist_new (3);
  gtk_widget_set_name (memList, "memList");
  gtk_object_set_data (GTK_OBJECT (memWin), "memList", memList);
  gtk_widget_show (memList);
  gtk_container_add (GTK_CONTAINER (memWin), memList);
  gtk_clist_set_column_width (GTK_CLIST (memList), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (memList), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (memList), 2, 80);
  gtk_clist_column_titles_show (GTK_CLIST (memList));

  memAddrLbl = gtk_label_new ("Osoite");
  gtk_widget_set_name (memAddrLbl, "memAddrLbl");
  gtk_object_set_data (GTK_OBJECT (memWin), "memAddrLbl", memAddrLbl);
  gtk_widget_show (memAddrLbl);
  gtk_clist_set_column_widget (GTK_CLIST (memList), 0, memAddrLbl);

  memNumVal = gtk_label_new ("Lukuarvo");
  gtk_widget_set_name (memNumVal, "memNumVal");
  gtk_object_set_data (GTK_OBJECT (memWin), "memNumVal", memNumVal);
  gtk_widget_show (memNumVal);
  gtk_clist_set_column_widget (GTK_CLIST (memList), 1, memNumVal);

  memIstrAddr = gtk_label_new ("Komento");
  gtk_widget_set_name (memIstrAddr, "memIstrAddr");
  gtk_object_set_data (GTK_OBJECT (memWin), "memIstrAddr", memIstrAddr);
  gtk_widget_show (memIstrAddr);
  gtk_clist_set_column_widget (GTK_CLIST (memList), 2, memIstrAddr);

  return memWin;
}

GtkWidget*
create_symWin ()
{
  GtkWidget *symWin;
  GtkWidget *symList;
  GtkWidget *symName;
  GtkWidget *symVal;

  symWin = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (symWin, "symWin");
  gtk_object_set_data (GTK_OBJECT (symWin), "symWin", symWin);
  gtk_widget_set_usize (symWin, 300, 150);
  gtk_signal_connect (GTK_OBJECT (symWin), "delete_event",
                      GTK_SIGNAL_FUNC (on_symWin_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (symWin), "Kntjn symbolit");
  gtk_window_set_policy (GTK_WINDOW (symWin), TRUE, TRUE, FALSE);

  symList = gtk_clist_new (2);
  gtk_widget_set_name (symList, "symList");
  gtk_object_set_data (GTK_OBJECT (symWin), "symList", symList);
  gtk_widget_show (symList);
  gtk_container_add (GTK_CONTAINER (symWin), symList);
  gtk_clist_set_column_width (GTK_CLIST (symList), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (symList), 1, 80);
  gtk_clist_column_titles_show (GTK_CLIST (symList));

  symName = gtk_label_new ("Nimi");
  gtk_widget_set_name (symName, "symName");
  gtk_object_set_data (GTK_OBJECT (symWin), "symName", symName);
  gtk_widget_show (symName);
  gtk_clist_set_column_widget (GTK_CLIST (symList), 0, symName);

  symVal = gtk_label_new ("Arvo");
  gtk_widget_set_name (symVal, "symVal");
  gtk_object_set_data (GTK_OBJECT (symWin), "symVal", symVal);
  gtk_widget_show (symVal);
  gtk_clist_set_column_widget (GTK_CLIST (symList), 1, symVal);

  return symWin;
}

GtkWidget*
create_kbdDialog ()
{
  GtkWidget *kbdDialog;
  GtkWidget *dialog_vbox1;
  GtkWidget *hbox2;
  GtkWidget *kbdPrompt;
  GtkWidget *kbdValue;
  GtkWidget *dialog_action_area1;
  GtkWidget *kbdOk;

  kbdDialog = gtk_dialog_new ();
  gtk_widget_set_name (kbdDialog, "kbdDialog");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "kbdDialog", kbdDialog);
  gtk_widget_set_usize (kbdDialog, 300, 120);
  gtk_window_set_title (GTK_WINDOW (kbdDialog), "KBD");
  gtk_window_set_policy (GTK_WINDOW (kbdDialog), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (kbdDialog)->vbox;
  gtk_widget_set_name (dialog_vbox1, "dialog_vbox1");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_set_name (hbox2, "hbox2");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "hbox2", hbox2);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), hbox2, TRUE, TRUE, 17);

  kbdPrompt = gtk_label_new ("Anna kokonaisluku:");
  gtk_widget_set_name (kbdPrompt, "kbdPrompt");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "kbdPrompt", kbdPrompt);
  gtk_widget_show (kbdPrompt);
  gtk_box_pack_start (GTK_BOX (hbox2), kbdPrompt, TRUE, TRUE, 0);

  kbdValue = gtk_entry_new ();
  gtk_widget_set_name (kbdValue, "kbdValue");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "kbdValue", kbdValue);
  gtk_widget_show (kbdValue);
  gtk_box_pack_start (GTK_BOX (hbox2), kbdValue, TRUE, TRUE, 0);
  gtk_widget_grab_focus (kbdValue);

  dialog_action_area1 = GTK_DIALOG (kbdDialog)->action_area;
  gtk_widget_set_name (dialog_action_area1, "dialog_action_area1");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  kbdOk = gtk_button_new_with_label ("OK");
  gtk_widget_set_name (kbdOk, "kbdOk");
  gtk_object_set_data (GTK_OBJECT (kbdDialog), "kbdOk", kbdOk);
  gtk_widget_show (kbdOk);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), kbdOk, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (kbdOk, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (kbdOk);
  gtk_signal_connect (GTK_OBJECT (kbdOk), "clicked",
                      GTK_SIGNAL_FUNC (on_kbdOk_clicked),
                      NULL);

  return kbdDialog;
}

GtkWidget*
create_crtDialog ()
{
  GtkWidget *crtDialog;
  GtkWidget *dialog_vbox2;
  GtkWidget *crtValue;
  GtkWidget *dialog_action_area2;
  GtkWidget *crtOK;
  GtkAcceleratorTable *accelerator_table;

  crtDialog = gtk_dialog_new ();
  gtk_widget_set_name (crtDialog, "crtDialog");
  gtk_object_set_data (GTK_OBJECT (crtDialog), "crtDialog", crtDialog);
  gtk_widget_set_usize (crtDialog, 200, 120);
  gtk_window_set_title (GTK_WINDOW (crtDialog), "CRT");
  gtk_window_set_policy (GTK_WINDOW (crtDialog), TRUE, TRUE, FALSE);

  dialog_vbox2 = GTK_DIALOG (crtDialog)->vbox;
  gtk_widget_set_name (dialog_vbox2, "dialog_vbox2");
  gtk_object_set_data (GTK_OBJECT (crtDialog), "dialog_vbox2", dialog_vbox2);
  gtk_widget_show (dialog_vbox2);

  crtValue = gtk_label_new ("140478");
  gtk_widget_set_name (crtValue, "crtValue");
  gtk_object_set_data (GTK_OBJECT (crtDialog), "crtValue", crtValue);
  gtk_widget_show (crtValue);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), crtValue, TRUE, TRUE, 0);

  dialog_action_area2 = GTK_DIALOG (crtDialog)->action_area;
  gtk_widget_set_name (dialog_action_area2, "dialog_action_area2");
  gtk_object_set_data (GTK_OBJECT (crtDialog), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);

  crtOK = gtk_button_new_with_label ("OK");
  gtk_widget_set_name (crtOK, "crtOK");
  gtk_object_set_data (GTK_OBJECT (crtDialog), "crtOK", crtOK);
  gtk_widget_show (crtOK);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), crtOK, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (crtOK, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (crtOK);
  gtk_signal_connect (GTK_OBJECT (crtOK), "clicked",
                      GTK_SIGNAL_FUNC (on_crtOK_clicked),
                      NULL);
  accelerator_table = gtk_accelerator_table_new ();
  gtk_window_add_accelerator_table (GTK_WINDOW (crtDialog), accelerator_table);
  gtk_widget_install_accelerator (crtOK, accelerator_table, "clicked",
                                  GDK_space, 0);

  return crtDialog;
}

GtkWidget*
create_errDialog ()
{
  GtkWidget *errDialog;
  GtkWidget *dialog_vbox3;
  GtkWidget *errValue;
  GtkWidget *dialog_action_area3;
  GtkWidget *errOK;

  errDialog = gtk_dialog_new ();
  gtk_widget_set_name (errDialog, "errDialog");
  gtk_object_set_data (GTK_OBJECT (errDialog), "errDialog", errDialog);
  gtk_widget_set_usize (errDialog, 200, 120);
  gtk_window_set_title (GTK_WINDOW (errDialog), "Virhe");
  gtk_window_set_policy (GTK_WINDOW (errDialog), TRUE, TRUE, FALSE);

  dialog_vbox3 = GTK_DIALOG (errDialog)->vbox;
  gtk_widget_set_name (dialog_vbox3, "dialog_vbox3");
  gtk_object_set_data (GTK_OBJECT (errDialog), "dialog_vbox3", dialog_vbox3);
  gtk_widget_show (dialog_vbox3);

  errValue = gtk_label_new ("On virhe ett tm nkyy!!");
  gtk_widget_set_name (errValue, "errValue");
  gtk_object_set_data (GTK_OBJECT (errDialog), "errValue", errValue);
  gtk_widget_show (errValue);
  gtk_box_pack_start (GTK_BOX (dialog_vbox3), errValue, TRUE, TRUE, 0);

  dialog_action_area3 = GTK_DIALOG (errDialog)->action_area;
  gtk_widget_set_name (dialog_action_area3, "dialog_action_area3");
  gtk_object_set_data (GTK_OBJECT (errDialog), "dialog_action_area3", dialog_action_area3);
  gtk_widget_show (dialog_action_area3);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area3), 10);

  errOK = gtk_button_new_with_label ("OK");
  gtk_widget_set_name (errOK, "errOK");
  gtk_object_set_data (GTK_OBJECT (errDialog), "errOK", errOK);
  gtk_widget_show (errOK);
  gtk_box_pack_start (GTK_BOX (dialog_action_area3), errOK, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (errOK, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (errOK);
  gtk_signal_connect (GTK_OBJECT (errOK), "clicked",
                      GTK_SIGNAL_FUNC (on_errOK_clicked),
                      NULL);

  return errDialog;
}

